@@ -1,7 +1,9 @@
 # quantalogic/utils/__init__.py
-from .exceptions import BreakException, ContinueException, ReturnException, WrappedException, has_await
-from .execution import AsyncExecutionResult, execute_async, interpret_ast, interpret_code
-from .function_utils import AsyncFunction, Function, LambdaFunction
+from .exceptions import BreakException, ContinueException, ReturnException, WrappedException
+from .utils import has_await
+from .execution import AsyncExecutionResult, execute_async, interpret_ast
+from .function_utils import AsyncFunction, Function, LambdaFunction, AsyncGeneratorFunction
+from .generator_wrapper import GeneratorWrapper
 from .interpreter_core import ASTInterpreter
 from .scope import Scope
 
@@ -9,7 +11,6 @@ __all__ = [
     'ASTInterpreter',
     'execute_async',
     'interpret_ast',
-    'interpret_code',
     'AsyncExecutionResult',
     'ReturnException',
     'BreakException',
@@ -19,5 +20,7 @@ __all__ = [
     'Function',
     'AsyncFunction',
     'LambdaFunction',
+    'AsyncGeneratorFunction',
+    'GeneratorWrapper',
     'Scope',
 ]
\ No newline at end of file
