@@ -12,57 +12,47 @@ async def visit_ListComp(interpreter, node: ast.ListComp, wrap_exceptions: bool
     if len(node.generators) == 1:
         comp = node.generators[0]
         iterable = await interpreter.visit(comp.iter, wrap_exceptions=wrap_exceptions)
-        logger.debug(f"Retrieved iterable for single generator: {iterable if iterable else 'None'}, type: {type(iterable) if iterable else 'NoneType'}")
-        logger.debug(f"Retrieved iterable of type {type(iterable)} for single generator, has __aiter__: {hasattr(iterable, '__aiter__')}, has __anext__: {hasattr(iterable, '__anext__')}")
+        # handle async iterables (async generators) and sync iterables
         if hasattr(iterable, '__aiter__'):
-            logger.debug("Starting async for loop for single generator")
-            try:
-                async for item in iterable:
-                    logger.debug(f"Starting async for loop for single generator, item: {item if item else 'None'}, type: {type(item) if item else 'NoneType'}")
-                    logger.debug(f"Starting iteration over item {item} from async iterable, item type: {type(item)}")
-                    new_frame = interpreter.env_stack[-1].copy()
-                    interpreter.env_stack.append(new_frame)
+            result = []
+            async for item in iterable:
+                new_frame = interpreter.env_stack[-1].copy()
+                interpreter.env_stack.append(new_frame)
+                try:
                     await interpreter.assign(comp.target, item)
-                    logger.debug(f"After assignment for single generator, scope keys: {list(interpreter.env_stack[-1].keys())}, target {comp.target.id if isinstance(comp.target, ast.Name) else 'complex target'} present: {comp.target.id in interpreter.env_stack[-1] if isinstance(comp.target, ast.Name) else 'N/A' }")
-                    logger.debug(f"Assigned item {item} to target for single generator, item type: {type(item)}")
-                    conditions = [await interpreter.visit(if_clause, wrap_exceptions=True) for if_clause in comp.ifs] if comp.ifs else [True]
-                    logger.debug(f"Conditions for single generator: {conditions}, all true? {all(conditions)}")
-                    if all(conditions):
-                        logger.debug(f"Evaluating elt for single generator, scope keys: {list(interpreter.env_stack[-1].keys())}")
-                        value = await interpreter.visit(node.elt, wrap_exceptions=wrap_exceptions)
-                        logger.debug(f"Element to append in list comp: {value}, type: {type(value)}")
-                        result.append(value)
-                        logger.debug(f"Appended value: {value}, type: {type(value)}, result_id: {id(result)}")
+                except TypeError:
                     interpreter.env_stack.pop()
-            except Exception as e:
-                logger.error(f"Exception in async for loop for single generator: {str(e)}")
-                raise
+                    continue
+                if comp.ifs:
+                    conds = [await interpreter.visit(if_clause, wrap_exceptions=True) for if_clause in comp.ifs]
+                    if not all(conds):
+                        interpreter.env_stack.pop()
+                        continue
+                val = await interpreter.visit(node.elt, wrap_exceptions=wrap_exceptions)
+                result.append(val)
+                interpreter.env_stack.pop()
+            interpreter.env_stack.pop()
+            return result
         else:
-            try:
-                logger.debug("Starting for loop for single generator")
-                for item in iterable:
-                    logger.debug(f"Processing item {item} from iterable, item type: {type(item)}")
-                    new_frame = interpreter.env_stack[-1].copy()
-                    interpreter.env_stack.append(new_frame)
+            result = []
+            for item in iterable:
+                new_frame = interpreter.env_stack[-1].copy()
+                interpreter.env_stack.append(new_frame)
+                try:
                     await interpreter.assign(comp.target, item)
-                    logger.debug(f"After assignment for single generator, scope keys: {list(interpreter.env_stack[-1].keys())}, target {comp.target.id if isinstance(comp.target, ast.Name) else 'complex target'} present: {comp.target.id in interpreter.env_stack[-1] if isinstance(comp.target, ast.Name) else 'N/A' }")
-                    logger.debug(f"Assigned item {item} to target for single generator, item type: {type(item)}")
-                    conditions = [await interpreter.visit(if_clause, wrap_exceptions=True) for if_clause in comp.ifs] if comp.ifs else [True]
-                    logger.debug(f"Conditions for single generator: {conditions}, all true? {all(conditions)}")
-                    if all(conditions):
-                        logger.debug(f"Evaluating elt for single generator, scope keys: {list(interpreter.env_stack[-1].keys())}")
-                        value = await interpreter.visit(node.elt, wrap_exceptions=wrap_exceptions)
-                        logger.debug(f"Element to append in list comp: {value}, type: {type(value)}")
-                        result.append(value)
-                        logger.debug(f"Appended value: {value}, type: {type(value)}, result_id: {id(result)}")
+                except TypeError:
                     interpreter.env_stack.pop()
-            except TypeError as e:
-                logger.debug(f"TypeError in for loop for iterable {iterable}: {e}")
-                lineno = getattr(node, "lineno", 1)
-                col = getattr(node, "col_offset", 0)
-                context_line = interpreter.source_lines[lineno - 1] if interpreter.source_lines and lineno <= len(interpreter.source_lines) else ""
-                from .exceptions import WrappedException
-                raise WrappedException(f"Object {iterable} is not iterable", e, lineno, col, context_line) from e
+                    continue
+                if comp.ifs:
+                    conds = [await interpreter.visit(if_clause, wrap_exceptions=True) for if_clause in comp.ifs]
+                    if not all(conds):
+                        interpreter.env_stack.pop()
+                        continue
+                val = await interpreter.visit(node.elt, wrap_exceptions=wrap_exceptions)
+                result.append(val)
+                interpreter.env_stack.pop()
+            interpreter.env_stack.pop()
+            return result
     else:
         async def rec(gen_idx: int):
             num_generators = len(node.generators)
@@ -73,11 +63,8 @@ async def visit_ListComp(interpreter, node: ast.ListComp, wrap_exceptions: bool
                 logger.debug(f"Evaluating elt for gen_idx {gen_idx}, scope keys: {list(interpreter.env_stack[-1].keys())}")
                 value = await interpreter.visit(node.elt, wrap_exceptions=wrap_exceptions)
                 logger.debug(f"Element to append in list comp: {value}, type: {type(value)}")
-                logger.debug(f"Debug: About to append element {value} with current scope keys: {list(interpreter.env_stack[-1].keys())}")
-                logger.debug(f"Appended value: {value}, type: {type(value)}, result_id: {id(result)}")
                 result.append(value)
-                logger.debug(f"After append, length: {len(result)}, result_id: {id(result)}")
-                logger.debug(f"Appending element to list comp result: {value}")
+                logger.debug(f"Appended value: {value}, type: {type(value)}, result_id: {id(result)}")
             else:
                 comp = node.generators[gen_idx]
                 iterable = await interpreter.visit(comp.iter, wrap_exceptions=wrap_exceptions)
@@ -91,7 +78,12 @@ async def visit_ListComp(interpreter, node: ast.ListComp, wrap_exceptions: bool
                             logger.debug(f"Starting iteration over item {item} from async iterable in generator {gen_idx}, item type: {type(item)}")
                             new_frame = interpreter.env_stack[-1].copy()
                             interpreter.env_stack.append(new_frame)
-                            await interpreter.assign(comp.target, item)
+                            try:
+                                await interpreter.assign(comp.target, item)
+                            except TypeError as e:
+                                logger.debug(f"Skipping async comprehension iteration for item {item}: {e}")
+                                interpreter.env_stack.pop()
+                                continue
                             logger.debug(f"After assignment for gen_idx {gen_idx}, scope keys: {list(interpreter.env_stack[-1].keys())}, target {comp.target.id if isinstance(comp.target, ast.Name) else 'complex target'} present: {comp.target.id in interpreter.env_stack[-1] if isinstance(comp.target, ast.Name) else 'N/A' }")
                             logger.debug(f"Assigned item {item} to target for gen_idx {gen_idx}, item type: {type(item)}")
                             conditions = [await interpreter.visit(if_clause, wrap_exceptions=True) for if_clause in comp.ifs] if comp.ifs else [True]
@@ -130,7 +122,7 @@ async def visit_ListComp(interpreter, node: ast.ListComp, wrap_exceptions: bool
                                 logger.debug(f"After calling rec with gen_idx {gen_idx + 1} for item {item}")
                             interpreter.env_stack.pop()
                     except TypeError as e:
-                        logger.debug(f"TypeError in async for loop for iterable {iterable}: {e}")
+                        logger.debug(f"TypeError in for loop for iterable {iterable}: {e}")
                         lineno = getattr(node, "lineno", 1)
                         col = getattr(node, "col_offset", 0)
                         context_line = interpreter.source_lines[lineno - 1] if interpreter.source_lines and lineno <= len(interpreter.source_lines) else ""
@@ -166,7 +158,12 @@ async def visit_DictComp(interpreter, node: ast.DictComp, wrap_exceptions: bool
                     logger.debug(f"Starting async for loop for gen_idx {gen_idx}, item: {item if item else 'None'}, type: {type(item) if item else 'NoneType'}")
                     new_frame = interpreter.env_stack[-1].copy()
                     interpreter.env_stack.append(new_frame)
-                    await interpreter.assign(comp.target, item)
+                    try:
+                        await interpreter.assign(comp.target, item)
+                    except TypeError as e:
+                        logger.debug(f"Skipping async comprehension iteration for item {item}: {e}")
+                        interpreter.env_stack.pop()
+                        continue
                     logger.debug(f"After assignment for gen_idx {gen_idx}, scope keys: {list(interpreter.env_stack[-1].keys())}, target {comp.target.id if isinstance(comp.target, ast.Name) else 'complex target'} present: {comp.target.id in interpreter.env_stack[-1] if isinstance(comp.target, ast.Name) else 'N/A' }")
                     logger.debug(f"Assigned item {item} to target for gen_idx {gen_idx}, item type: {type(item)}")
                     conditions = [await interpreter.visit(if_clause, wrap_exceptions=True) for if_clause in comp.ifs]
@@ -224,7 +221,12 @@ async def visit_SetComp(interpreter, node: ast.SetComp, wrap_exceptions: bool =
                     logger.debug(f"Starting async for loop for gen_idx {gen_idx}, item: {item if item else 'None'}, type: {type(item) if item else 'NoneType'}")
                     new_frame = interpreter.env_stack[-1].copy()
                     interpreter.env_stack.append(new_frame)
-                    await interpreter.assign(comp.target, item)
+                    try:
+                        await interpreter.assign(comp.target, item)
+                    except TypeError as e:
+                        logger.debug(f"Skipping async comprehension iteration for item {item}: {e}")
+                        interpreter.env_stack.pop()
+                        continue
                     logger.debug(f"After assignment for gen_idx {gen_idx}, scope keys: {list(interpreter.env_stack[-1].keys())}, target {comp.target.id if isinstance(comp.target, ast.Name) else 'complex target'} present: {comp.target.id in interpreter.env_stack[-1] if isinstance(comp.target, ast.Name) else 'N/A' }")
                     logger.debug(f"Assigned item {item} to target for gen_idx {gen_idx}, item type: {type(item)}")
                     conditions = [await interpreter.visit(if_clause, wrap_exceptions=True) for if_clause in comp.ifs]
@@ -289,7 +291,12 @@ async def visit_GeneratorExp(interpreter, node: ast.GeneratorExp, wrap_exception
                         logger.debug(f"Starting iteration over item {item} from async iterable in generator {gen_idx}, item type: {type(item)}")
                         new_frame = interpreter.env_stack[-1].copy()
                         interpreter.env_stack.append(new_frame)
-                        await interpreter.assign(comp.target, item)
+                        try:
+                            await interpreter.assign(comp.target, item)
+                        except TypeError as e:
+                            logger.debug(f"Skipping async comprehension iteration for item {item}: {e}")
+                            interpreter.env_stack.pop()
+                            continue
                         logger.debug(f"After assignment for gen_idx {gen_idx}, scope keys: {list(interpreter.env_stack[-1].keys())}, target {comp.target.id if isinstance(comp.target, ast.Name) else 'complex target'} present: {comp.target.id in interpreter.env_stack[-1] if isinstance(comp.target, ast.Name) else 'N/A' }")
                         logger.debug(f"Assigned item {item} to target for gen_idx {gen_idx}, item type: {type(item)}")
                         conditions = [await interpreter.visit(if_clause, wrap_exceptions=True) for if_clause in comp.ifs]
@@ -348,7 +355,12 @@ async def visit_GeneratorExp(interpreter, node: ast.GeneratorExp, wrap_exception
                             logger.debug(f"Starting async for loop for gen_idx 0, item: {item if item else 'None'}, type: {type(item) if item else 'NoneType'}")
                             new_frame = interpreter.env_stack[-1].copy()
                             interpreter.env_stack.append(new_frame)
-                            await interpreter.assign(comp.target, item)
+                            try:
+                                await interpreter.assign(comp.target, item)
+                            except TypeError as e:
+                                logger.debug(f"Skipping async comprehension iteration for item {item}: {e}")
+                                interpreter.env_stack.pop()
+                                continue
                             logger.debug(f"After assignment for gen_idx 0, scope keys: {list(interpreter.env_stack[-1].keys())}, target {comp.target.id if isinstance(comp.target, ast.Name) else 'complex target'} present: {comp.target.id in interpreter.env_stack[-1] if isinstance(comp.target, ast.Name) else 'N/A' }")
                             logger.debug(f"Assigned item {item} to target for gen_idx 0, item type: {type(item)}")
                             conditions = [await interpreter.visit(if_clause, wrap_exceptions=True) for if_clause in comp.ifs]
@@ -405,7 +417,12 @@ async def visit_GeneratorExp(interpreter, node: ast.GeneratorExp, wrap_exception
                                     logger.debug(f"Starting async for loop for gen_idx {gen_idx}, item: {item if item else 'None'}, type: {type(item) if item else 'NoneType'}")
                                     new_frame = interpreter.env_stack[-1].copy()
                                     interpreter.env_stack.append(new_frame)
-                                    await interpreter.assign(comp.target, item)
+                                    try:
+                                        await interpreter.assign(comp.target, item)
+                                    except TypeError as e:
+                                        logger.debug(f"Skipping async comprehension iteration for item {item}: {e}")
+                                        interpreter.env_stack.pop()
+                                        continue
                                     logger.debug(f"After assignment for gen_idx {gen_idx}, scope keys: {list(interpreter.env_stack[-1].keys())}, target {comp.target.id if isinstance(comp.target, ast.Name) else 'complex target'} present: {comp.target.id in interpreter.env_stack[-1] if isinstance(comp.target, ast.Name) else 'N/A' }")
                                     logger.debug(f"Assigned item {item} to target for gen_idx {gen_idx}, item type: {type(item)}")
                                     conditions = [await interpreter.visit(if_clause, wrap_exceptions=True) for if_clause in comp.ifs]
